package com.franckbarbier.Beverage_vending_machine;

public class Beverage_vending_machine extends com.pauware.pauware_engine.Core.AbstractTimer_monitor implements java.io.Serializable {

    private static final long serialVersionUID = 1L;
    /**
     * UML states
     */
    private com.pauware.pauware_engine.Core.AbstractState _Coin_insertion;
    private com.pauware.pauware_engine.Core.AbstractState _Service;
    private com.pauware.pauware_engine.Core.AbstractState _Pick_up;
    private com.pauware.pauware_engine.Core.AbstractStateMachine _BVM_state_machine;
    /**
     * UML variables
     */
    private Integer _beverage_price;
    private Integer _total;

    private void init_structure(Integer beverage_price) throws com.pauware.pauware_engine.Exceptions.State_exception {
        _beverage_price = beverage_price;
        _total = Integer.valueOf(0);
    }

    private void init_behavior() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Coin_insertion = new com.pauware.pauware_engine.Core.State("Coin insertion");
        _Coin_insertion.inputState();
        _Service = new com.pauware.pauware_engine.Core.State("Service");
        _Service.set_entryAction(this, "to_be_set", new Object[]{Long.valueOf(1000L)});
        _Service.set_exitAction(this, "to_be_killed");
        _Pick_up = new com.pauware.pauware_engine.Core.State("Pick up");
        _BVM_state_machine = new com.pauware.pauware_engine.Core.StateMachine(_Coin_insertion.xor(_Service).xor(_Pick_up), "Beverage vending machine", com.pauware.pauware_engine.Core.AbstractStateMachine.Don_t_show_on_system_out, new com.pauware.pauware2web.PauWare2Web_client());
    }

    public void start() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _BVM_state_machine.fires("choose", _Coin_insertion, _Service, this, "required", this, "please_wait_for_beverage");
        _BVM_state_machine.fires("choose", _Coin_insertion, _Service, this, "required", this, "decrease_total");
        /**
         * Fake arguments for 'coin_insertion'. This allows the visualization of
         * this transition by PauWare view
         */
        _BVM_state_machine.fires("coin_insertion", _Coin_insertion, _Coin_insertion, true, this, "increase_total", new Object[]{Integer.valueOf(0)});
        /**
         * End of fake arguments
         */
        _BVM_state_machine.fires("time_out", _Service, _Pick_up, true, this, "give_change");
        _BVM_state_machine.fires("time_out", _Service, _Pick_up, true, this, "take_beverage");
        _BVM_state_machine.fires("pick_up", _Pick_up, _Coin_insertion);
        _BVM_state_machine.fires("cancel", _Coin_insertion, _Coin_insertion, true, this, "give_change");
        _BVM_state_machine.start();
    }

    public void stop() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _BVM_state_machine.stop();
    }

    public Beverage_vending_machine(Integer beverage_price) throws com.pauware.pauware_engine.Exceptions.State_exception {
        init_structure(beverage_price);
        init_behavior();
    }

    public void cancel() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _BVM_state_machine.run_to_completion("cancel");
    }

    public void coin_insertion(Integer coin) throws com.pauware.pauware_engine.Exceptions.State_exception {
        _BVM_state_machine.fires("coin_insertion", _Coin_insertion, _Coin_insertion, true, this, "increase_total", new Object[]{coin});
        _BVM_state_machine.run_to_completion("coin_insertion");
    }

    public void choose() throws com.pauware.pauware_engine.Exceptions.State_exception {
//        _BVM_state_machine.fires("choose", _Coin_insertion, _Service, required(), this, "please_wait_for_beverage"); // this code works here
//        _BVM_state_machine.fires("choose", _Coin_insertion, _Service, required(), this, "decrease_total"); // this code works here
        _BVM_state_machine.run_to_completion("choose");
    }

    public void pick_up() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _BVM_state_machine.run_to_completion("pick_up");
    }

    public void time_out(long delay, com.pauware.pauware_engine.Core.AbstractState context) throws com.pauware.pauware_engine.Exceptions.State_exception {
        assert (delay == 1000L); // Check for what delay the timer event is received...
        assert (context.name().equals("Service"));
        _BVM_state_machine.run_to_completion("time_out");
    }

    public void time_out_error(com.pauware.pauware_engine.Exceptions.State_exception se) throws com.pauware.pauware_engine.Exceptions.State_exception {
        se.printStackTrace();
        // possible fault recovery here...
    }

    /**
     * UML guards
     */
    public boolean required() {
        return _total.intValue() >= _beverage_price.intValue();
    }

    /**
     * UML actions
     */
    public Integer decrease_total() {
        _total = Integer.valueOf(_total.intValue() - _beverage_price.intValue());
        return _total;
    }

    public Integer increase_total(Integer coin) {
        _total = Integer.valueOf(_total.intValue() + coin.intValue());
        return _total;
    }

    public void give_change() {
        System.out.println("\t\tGive change: " + _total + '\n');
        _total = Integer.valueOf(0);
    }

    public void please_wait_for_beverage() {
        System.out.println("\t\tPlease wait for beverage\n");
    }

    public void take_beverage() {
        System.out.println("\t\tTake beverage\n");
    }

    public static void main(String args[]) {
        try {
            Beverage_vending_machine beverage_vending_machine = new Beverage_vending_machine(Integer.valueOf(30));
            beverage_vending_machine.start();

            beverage_vending_machine.coin_insertion(Integer.valueOf(10));
            beverage_vending_machine.coin_insertion(Integer.valueOf(10));
            beverage_vending_machine.coin_insertion(Integer.valueOf(20));
            beverage_vending_machine.choose();
            Thread.sleep(1500); // 'time_out' occurs after 1 sec.
            beverage_vending_machine.pick_up();

            beverage_vending_machine.coin_insertion(Integer.valueOf(10));
            beverage_vending_machine.coin_insertion(Integer.valueOf(10));
            beverage_vending_machine.coin_insertion(Integer.valueOf(20));
            beverage_vending_machine.choose();
            Thread.sleep(1500); // 'time_out' occurs after 1 sec.
            beverage_vending_machine.pick_up();

            beverage_vending_machine.coin_insertion(Integer.valueOf(10));
            beverage_vending_machine.coin_insertion(Integer.valueOf(10));
            beverage_vending_machine.cancel();
//
            beverage_vending_machine.stop();
            /**
             * For testing purposes only
             */
//            java.io.FileOutputStream fileOut = new java.io.FileOutputStream("BVM.ser");
//            java.io.ObjectOutputStream out = new java.io.ObjectOutputStream(fileOut);
//            out.writeObject(beverage_vending_machine);
//            out.close();
//            fileOut.close();
//
//            java.io.FileInputStream fileIn = new java.io.FileInputStream("BVM.ser");
//            java.io.ObjectInputStream in = new java.io.ObjectInputStream(fileIn);
//            beverage_vending_machine = (Beverage_vending_machine) in.readObject();
//            in.close();
//            fileIn.close();
            /**
             * End of testing
             */
        } catch (Throwable t) {
            t.printStackTrace();
        }
    }
}
